/*
* Copyright (c) 2016-2019 Irlan Robson https://irlanrobson.github.io
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#ifndef VIEW_H
#define VIEW_H

#include <bounce/common/math/vec2.h>

struct GLFWwindow;

class ViewModel;

class View
{	
public:
	View(GLFWwindow* window);
	~View();

	void Event_SetWindowSize(int w, int h);
	void Event_Press_Key(int button);
	void Event_Release_Key(int button);
	void Event_Press_Mouse(int button);
	void Event_Release_Mouse(int button);
	void Event_Move_Cursor(float x, float y);
	void Event_Scroll(float dx, float dy);

	void BeginInterface();
	void Interface();
	void InterfaceProfileTree();
	void InterfaceProfileTreeStats();
	void EndInterface();
private:
	friend class ViewModel;

	b3Vec2 GetCursorPosition() const;

	ViewModel* m_viewModel;

	GLFWwindow* m_window;
	b3Vec2 m_ps0;
};

#endif